@@ -27,6 +27,8 @@ import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 import java.util.stream.Collectors;
 
+import javax.annotation.concurrent.NotThreadSafe;
+
 import com.google.common.annotations.VisibleForTesting;
 import com.google.common.collect.Iterables;
 
@@ -53,6 +55,7 @@ import static org.apache.cassandra.utils.Throwables.merge;
  *
  * @see LogTransaction
  */
+@NotThreadSafe
 final class LogFile implements AutoCloseable
 {
     private static final Logger logger = LoggerFactory.getLogger(LogFile.class);
@@ -66,10 +69,9 @@ final class LogFile implements AutoCloseable
     private final LogReplicaSet replicas = new LogReplicaSet();
 
     // The transaction records, this set must be ORDER PRESERVING
-    private final LinkedHashSet<LogRecord> records = new LinkedHashSet<>();
-    // the transaction records we have written to disk - used to guarantee that the
-    // on-disk log files become identical when creating a new replica
-    private final LinkedHashSet<LogRecord> onDiskRecords = new LinkedHashSet<>();
+    private final Set<LogRecord> records = new LinkedHashSet<>();
+    private final Set<LogRecord> onDiskRecords = new LinkedHashSet<>();
+    private boolean completed = false;
 
     // The type of the transaction
     private final OperationType type;
@@ -125,6 +127,11 @@ final class LogFile implements AutoCloseable
 
             deleteFilesForRecordsOfType(committed() ? Type.REMOVE : Type.ADD);
 
+            // safe to release memory for both types of records, the completed flag will prevent
+            // new records being added.
+            records.clear();
+            onDiskRecords.clear();
+
             // we sync the parent directories between contents and log deletion
             // to ensure there is a happens before edge between them
             Throwables.maybeFail(syncDirectory(accumulate));
@@ -164,6 +171,11 @@ final class LogFile implements AutoCloseable
             logger.error("Failed to read records for transaction log {}", this);
             return false;
         }
+        LogRecord lastRecord = getLastRecord();
+        if (lastRecord != null &&
+            (lastRecord.type == Type.COMMIT || lastRecord.type == Type.ABORT) &&
+            lastRecord.isValid())
+            completed = true;
 
         Set<String> absolutePaths = new HashSet<>();
         for (LogRecord record : records)
@@ -278,11 +290,13 @@ final class LogFile implements AutoCloseable
     void commit()
     {
         addRecord(LogRecord.makeCommit(System.currentTimeMillis()));
+        completed = true;
     }
 
     void abort()
     {
         addRecord(LogRecord.makeAbort(System.currentTimeMillis()));
+        completed = true;
     }
 
     private boolean isLastRecordValidWithType(Type type)
@@ -298,14 +312,9 @@ final class LogFile implements AutoCloseable
         return isLastRecordValidWithType(Type.COMMIT);
     }
 
-    boolean aborted()
-    {
-        return isLastRecordValidWithType(Type.ABORT);
-    }
-
     boolean completed()
     {
-        return committed() || aborted();
+        return completed;
     }
 
     void add(SSTable table)
@@ -355,8 +364,8 @@ final class LogFile implements AutoCloseable
 
     void addRecord(LogRecord record)
     {
-        if (completed())
-            throw new IllegalStateException("Transaction already completed");
+        if (completed)
+            throw TransactionAlreadyCompletedException.create(getFiles());
 
         if (records.contains(record))
             throw new IllegalStateException("Record already exists");
@@ -369,6 +378,9 @@ final class LogFile implements AutoCloseable
 
     void remove(SSTable table)
     {
+        if (completed)
+            throw TransactionAlreadyCompletedException.create(getFiles());
+
         LogRecord record = makeAddRecord(table);
         assert records.contains(record) : String.format("[%s] is not tracked by %s", record, id);
         assert record.absolutePath.isPresent();
@@ -403,8 +415,6 @@ final class LogFile implements AutoCloseable
 
         for (List<File> toDelete : existingFiles.values())
             LogFile.deleteRecordFiles(toDelete);
-
-        records.clear();
     }
 
     private static void deleteRecordFiles(List<File> existingFiles)
