@@ -28,25 +28,19 @@ import java.util.List;
 import java.util.Map;
 import java.util.Objects;
 
-import org.slf4j.Logger;
-import org.slf4j.LoggerFactory;
-
 import org.apache.cassandra.cql3.AssignmentTestable;
 import org.apache.cassandra.cql3.CQL3Type;
 import org.apache.cassandra.cql3.ColumnSpecification;
 import org.apache.cassandra.cql3.Term;
-import org.apache.cassandra.db.TypeSizes;
 import org.apache.cassandra.db.rows.Cell;
 import org.apache.cassandra.exceptions.SyntaxException;
-import org.apache.cassandra.serializers.TypeSerializer;
-import org.apache.cassandra.serializers.MarshalException;
-
-import org.apache.cassandra.transport.ProtocolVersion;
-import org.apache.cassandra.utils.FastByteOperations;
-import org.github.jamm.Unmetered;
-import org.apache.cassandra.io.util.DataOutputPlus;
 import org.apache.cassandra.io.util.DataInputPlus;
+import org.apache.cassandra.io.util.DataOutputPlus;
+import org.apache.cassandra.serializers.MarshalException;
+import org.apache.cassandra.serializers.TypeSerializer;
+import org.apache.cassandra.transport.ProtocolVersion;
 import org.apache.cassandra.utils.ByteBufferUtil;
+import org.github.jamm.Unmetered;
 
 import static org.apache.cassandra.db.marshal.AbstractType.ComparisonType.CUSTOM;
 
@@ -61,8 +55,6 @@ import static org.apache.cassandra.db.marshal.AbstractType.ComparisonType.CUSTOM
 @Unmetered
 public abstract class AbstractType<T> implements Comparator<ByteBuffer>, AssignmentTestable
 {
-    private static final Logger logger = LoggerFactory.getLogger(AbstractType.class);
-
     public final Comparator<ByteBuffer> reverseComparator;
 
     public enum ComparisonType
@@ -85,6 +77,7 @@ public abstract class AbstractType<T> implements Comparator<ByteBuffer>, Assignm
 
     public final ComparisonType comparisonType;
     public final boolean isByteOrderComparable;
+    public final ValueComparators comparatorSet;
 
     protected AbstractType(ComparisonType comparisonType)
     {
@@ -93,7 +86,7 @@ public abstract class AbstractType<T> implements Comparator<ByteBuffer>, Assignm
         reverseComparator = (o1, o2) -> AbstractType.this.compare(o2, o1);
         try
         {
-            Method custom = getClass().getMethod("compareCustom", ByteBuffer.class, ByteBuffer.class);
+            Method custom = getClass().getMethod("compareCustom", Object.class, ValueAccessor.class, Object.class, ValueAccessor.class);
             if ((custom.getDeclaringClass() == AbstractType.class) == (comparisonType == CUSTOM))
                 throw new IllegalStateException((comparisonType == CUSTOM ? "compareCustom must be overridden if ComparisonType is CUSTOM"
                                                                          : "compareCustom should not be overridden if ComparisonType is not CUSTOM")
@@ -103,6 +96,17 @@ public abstract class AbstractType<T> implements Comparator<ByteBuffer>, Assignm
         {
             throw new IllegalStateException();
         }
+
+        comparatorSet = new ValueComparators((l, r) -> compare(l, ByteArrayAccessor.instance, r, ByteArrayAccessor.instance),
+                                             (l, r) -> compare(l, ByteBufferAccessor.instance, r, ByteBufferAccessor.instance));
+    }
+
+    static <VL, VR, T extends Comparable<T>> int compareComposed(VL left, ValueAccessor<VL> accessorL, VR right, ValueAccessor<VR> accessorR, AbstractType<T> type)
+    {
+        if (accessorL.isEmpty(left) || accessorR.isEmpty(right))
+            return Boolean.compare(accessorR.isEmpty(right), accessorL.isEmpty(left));
+
+        return type.compose(left, accessorL).compareTo(type.compose(right, accessorR));
     }
 
     public static List<String> asCQLTypeStringList(List<AbstractType<?>> abstractTypes)
@@ -113,26 +117,36 @@ public abstract class AbstractType<T> implements Comparator<ByteBuffer>, Assignm
         return r;
     }
 
-    public T compose(ByteBuffer bytes)
+    public final T compose(ByteBuffer bytes)
     {
         return getSerializer().deserialize(bytes);
     }
 
+    public <V> T compose(V value, ValueAccessor<V> accessor)
+    {
+        return getSerializer().deserialize(value, accessor);
+    }
+
     public ByteBuffer decompose(T value)
     {
         return getSerializer().serialize(value);
     }
 
     /** get a string representation of the bytes used for various identifier (NOT just for log messages) */
-    public String getString(ByteBuffer bytes)
+    public <V> String getString(V value, ValueAccessor<V> accessor)
     {
-        if (bytes == null)
+        if (value == null)
             return "null";
 
         TypeSerializer<T> serializer = getSerializer();
-        serializer.validate(bytes);
+        serializer.validate(value, accessor);
 
-        return serializer.toString(serializer.deserialize(bytes));
+        return serializer.toString(serializer.deserialize(value, accessor));
+    }
+
+    public final String getString(ByteBuffer bytes)
+    {
+        return getString(bytes, ByteBufferAccessor.instance);
     }
 
     /** get a byte representation of the given string. */
@@ -158,29 +172,46 @@ public abstract class AbstractType<T> implements Comparator<ByteBuffer>, Assignm
         return '"' + Objects.toString(getSerializer().deserialize(buffer), "") + '"';
     }
 
+    public <V> String toJSONString(V value, ValueAccessor<V> accessor, ProtocolVersion protocolVersion)
+    {
+        return toJSONString(accessor.toBuffer(value), protocolVersion); // FIXME
+    }
+
     /* validate that the byte array is a valid sequence for the type we are supposed to be comparing */
     public void validate(ByteBuffer bytes) throws MarshalException
     {
-        getSerializer().validate(bytes);
+        validate(bytes, ByteBufferAccessor.instance);
+    }
+
+    public <V> void validate(V value, ValueAccessor<V> accessor) throws MarshalException
+    {
+        getSerializer().validate(value, accessor);
     }
 
     public final int compare(ByteBuffer left, ByteBuffer right)
     {
-        return isByteOrderComparable
-               ? FastByteOperations.compareUnsigned(left, right)
-               : compareCustom(left, right);
+        return compare(left, ByteBufferAccessor.instance, right, ByteBufferAccessor.instance);
+    }
+
+    public final <VL, VR> int compare(VL left, ValueAccessor<VL> accessorL, VR right, ValueAccessor<VR> accessorR)
+    {
+        return isByteOrderComparable ? ValueAccessor.compare(left, accessorL, right, accessorR) : compareCustom(left, accessorL, right, accessorR);
     }
 
     /**
      * Implement IFF ComparisonType is CUSTOM
      *
-     * Compares the ByteBuffer representation of two instances of this class,
+     * Compares the byte representation of two instances of this class,
      * for types where this cannot be done by simple in-order comparison of the
      * unsigned bytes
      *
      * Standard Java compare semantics
+     * @param left
+     * @param accessorL
+     * @param right
+     * @param accessorR
      */
-    public int compareCustom(ByteBuffer left, ByteBuffer right)
+    public <VL, VR> int compareCustom(VL left, ValueAccessor<VL> accessorL, VR right, ValueAccessor<VR> accessorR)
     {
         throw new UnsupportedOperationException();
     }
@@ -193,9 +224,9 @@ public abstract class AbstractType<T> implements Comparator<ByteBuffer>, Assignm
      * @param cellValue ByteBuffer representing cell value
      * @throws MarshalException
      */
-    public void validateCellValue(ByteBuffer cellValue) throws MarshalException
+    public <V> void validateCellValue(V cellValue, ValueAccessor<V> accessor) throws MarshalException
     {
-        validate(cellValue);
+        validate(cellValue, accessor);
     }
 
     /* Most of our internal type should override that. */
@@ -319,19 +350,14 @@ public abstract class AbstractType<T> implements Comparator<ByteBuffer>, Assignm
      *
      * Unless you're doing something very similar to CollectionsType, you shouldn't override this.
      */
-    public int compareCollectionMembers(ByteBuffer v1, ByteBuffer v2, ByteBuffer collectionName)
+    public <VL, VR> int compareCollectionMembers(VL left, ValueAccessor<VL> accessorL, VR right, ValueAccessor<VR> accessorR, VL collectionName)
     {
-        return compare(v1, v2);
+        return compare(left, accessorL, right, accessorR);
     }
 
-    /**
-     * An alternative validation function used by CollectionsType in conjunction with CompositeType.
-     *
-     * This is similar to the compare function above.
-     */
-    public void validateCollectionMember(ByteBuffer bytes, ByteBuffer collectionName) throws MarshalException
+    public <V> void validateCollectionMember(V value, V collectionName, ValueAccessor<V> accessor) throws MarshalException
     {
-        validate(bytes);
+        getSerializer().validate(value, accessor);
     }
 
     public boolean isCollection()
@@ -364,6 +390,11 @@ public abstract class AbstractType<T> implements Comparator<ByteBuffer>, Assignm
         return this;
     }
 
+    public List<AbstractType<?>> subTypes()
+    {
+        return Collections.emptyList();
+    }
+
     /**
      * Returns an AbstractType instance that is equivalent to this one, but with all nested UDTs and collections
      * explicitly frozen.
@@ -393,15 +424,6 @@ public abstract class AbstractType<T> implements Comparator<ByteBuffer>, Assignm
         return this.toString();
     }
 
-    /**
-     * The number of subcomponents this type has.
-     * This is always 1, i.e. the type has only itself as "subcomponents", except for CompositeType.
-     */
-    public int componentsCount()
-    {
-        return 1;
-    }
-
     /**
      * Return a list of the "subcomponents" this type has.
      * This always return a singleton list with the type itself except for CompositeType.
@@ -414,56 +436,71 @@ public abstract class AbstractType<T> implements Comparator<ByteBuffer>, Assignm
     /**
      * The length of values for this type if all values are of fixed length, -1 otherwise.
      */
-    protected int valueLengthIfFixed()
+    public int valueLengthIfFixed()
     {
         return -1;
     }
 
-    public void validateIfFixedSize(ByteBuffer value)
-    {
-        if (valueLengthIfFixed() < 0)
-            return;
-
-        validate(value);
-    }
-
     // This assumes that no empty values are passed
     public void writeValue(ByteBuffer value, DataOutputPlus out) throws IOException
     {
-        assert value.hasRemaining();
-        int valueLengthIfFixed = valueLengthIfFixed();
-        assert valueLengthIfFixed < 0 || value.remaining() == valueLengthIfFixed : String.format("Expected exactly %d bytes, but was %d",
-                                                                                                 valueLengthIfFixed, value.remaining());
+        writeValue(value, ByteBufferAccessor.instance, out);
+    }
 
-        if (valueLengthIfFixed >= 0)
-            out.write(value);
+    // This assumes that no empty values are passed
+    public  <V> void writeValue(V value, ValueAccessor<V> accessor, DataOutputPlus out) throws IOException
+    {
+        assert !accessor.isEmpty(value) : "bytes should not be empty for type " + this;
+        int expectedValueLength = valueLengthIfFixed();
+        if (expectedValueLength >= 0)
+        {
+            int actualValueLength = accessor.size(value);
+            if (actualValueLength == expectedValueLength)
+                accessor.write(value, out);
+            else
+                throw new IOException(String.format("Expected exactly %d bytes, but was %d",
+                                                    expectedValueLength, actualValueLength));
+        }
         else
-            ByteBufferUtil.writeWithVIntLength(value, out);
+        {
+            accessor.writeWithVIntLength(value, out);
+        }
     }
 
     public long writtenLength(ByteBuffer value)
     {
-        assert value.hasRemaining();
-        int valueLengthIfFixed = valueLengthIfFixed();
-        assert valueLengthIfFixed < 0 || value.remaining() == valueLengthIfFixed : String.format("Expected exactly %d bytes, but was %d",
-                                                                                                 valueLengthIfFixed, value.remaining());
-
-        return valueLengthIfFixed >= 0
-             ? value.remaining()
-             : TypeSizes.sizeofWithVIntLength(value);
+        return writtenLength(value, ByteBufferAccessor.instance);
     }
 
-    public ByteBuffer readValue(DataInputPlus in) throws IOException
+    public <V> long writtenLength(V value, ValueAccessor<V> accessor)
     {
-        return readValue(in, Integer.MAX_VALUE);
+        assert !accessor.isEmpty(value) : "bytes should not be empty for type " + this;
+        return valueLengthIfFixed() >= 0
+               ? accessor.size(value) // if the size is wrong, this will be detected in writeValue
+               : accessor.sizeWithVIntLength(value);
     }
 
-    public ByteBuffer readValue(DataInputPlus in, int maxValueSize) throws IOException
+    public ByteBuffer readBuffer(DataInputPlus in) throws IOException
+    {
+        return readBuffer(in, Integer.MAX_VALUE);
+    }
+
+    public ByteBuffer readBuffer(DataInputPlus in, int maxValueSize) throws IOException
+    {
+        return read(ByteBufferAccessor.instance, in, maxValueSize);
+    }
+
+    public byte[] readArray(DataInputPlus in, int maxValueSize) throws IOException
+    {
+        return read(ByteArrayAccessor.instance, in, maxValueSize);
+    }
+
+    public <V> V read(ValueAccessor<V> accessor, DataInputPlus in, int maxValueSize) throws IOException
     {
         int length = valueLengthIfFixed();
 
         if (length >= 0)
-            return ByteBufferUtil.read(in, length);
+            return accessor.read(in, length);
         else
         {
             int l = (int)in.readUnsignedVInt();
@@ -475,7 +512,7 @@ public abstract class AbstractType<T> implements Comparator<ByteBuffer>, Assignm
                                                     "which is set via max_value_size_in_mb in cassandra.yaml",
                                                     l, maxValueSize));
 
-            return ByteBufferUtil.read(in, l);
+            return accessor.read(in, l);
         }
     }
 
@@ -488,16 +525,64 @@ public abstract class AbstractType<T> implements Comparator<ByteBuffer>, Assignm
             ByteBufferUtil.skipWithVIntLength(in);
     }
 
-    public boolean referencesUserType(String userTypeName)
+    public final boolean referencesUserType(ByteBuffer name)
+    {
+        return referencesUserType(name, ByteBufferAccessor.instance);
+    }
+
+    public <V> boolean referencesUserType(V name, ValueAccessor<V> accessor)
     {
         return false;
     }
 
+    /**
+     * Returns an instance of this type with all references to the provided user type recursively replaced with its new
+     * definition.
+     */
+    public AbstractType<?> withUpdatedUserType(UserType udt)
+    {
+        return this;
+    }
+
+    /**
+     * Replace any instances of UserType with equivalent TupleType-s.
+     *
+     * We need it for dropped_columns, to allow safely dropping unused user types later without retaining any references
+     * to them in system_schema.dropped_columns.
+     */
+    public AbstractType<?> expandUserTypes()
+    {
+        return this;
+    }
+
     public boolean referencesDuration()
     {
         return false;
     }
 
+    /**
+     * Tests whether a CQL value having this type can be assigned to the provided receiver.
+     */
+    public AssignmentTestable.TestResult testAssignment(AbstractType<?> receiverType)
+    {
+        // testAssignement is for CQL literals and native protocol values, none of which make a meaningful
+        // difference between frozen or not and reversed or not.
+
+        if (isFreezable() && !isMultiCell())
+            receiverType = receiverType.freeze();
+
+        if (isReversed() && !receiverType.isReversed())
+            receiverType = ReversedType.getInstance(receiverType);
+
+        if (equals(receiverType))
+            return AssignmentTestable.TestResult.EXACT_MATCH;
+
+        if (receiverType.isValueCompatibleWith(this))
+            return AssignmentTestable.TestResult.WEAKLY_ASSIGNABLE;
+
+        return AssignmentTestable.TestResult.NOT_ASSIGNABLE;
+    }
+
     /**
      * This must be overriden by subclasses if necessary so that for any
      * AbstractType, this == TypeParser.parse(toString()).
@@ -511,17 +596,6 @@ public abstract class AbstractType<T> implements Comparator<ByteBuffer>, Assignm
         return getClass().getName();
     }
 
-    /**
-     * Checks to see if two types are equal when ignoring or not ignoring differences in being frozen, depending on
-     * the value of the ignoreFreezing parameter.
-     * @param other type to compare
-     * @param ignoreFreezing if true, differences in the types being frozen will be ignored
-     */
-    public boolean equals(Object other, boolean ignoreFreezing)
-    {
-        return this.equals(other);
-    }
-
     public void checkComparable()
     {
         switch (comparisonType)
@@ -533,21 +607,6 @@ public abstract class AbstractType<T> implements Comparator<ByteBuffer>, Assignm
 
     public final AssignmentTestable.TestResult testAssignment(String keyspace, ColumnSpecification receiver)
     {
-        // We should ignore the fact that the output type is frozen in our comparison as functions do not support
-        // frozen types for arguments
-        AbstractType<?> receiverType = receiver.type;
-        if (isFreezable() && !isMultiCell())
-            receiverType = receiverType.freeze();
-
-        if (isReversed())
-            receiverType = ReversedType.getInstance(receiverType);
-
-        if (equals(receiverType))
-            return AssignmentTestable.TestResult.EXACT_MATCH;
-
-        if (receiverType.isValueCompatibleWith(this))
-            return AssignmentTestable.TestResult.WEAKLY_ASSIGNABLE;
-
-        return AssignmentTestable.TestResult.NOT_ASSIGNABLE;
+        return testAssignment(receiver.type);
     }
 }
