@@ -1237,7 +1237,7 @@ public class TableMetadata implements SchemaElement
                 DroppedColumn dropped = iterDropped.next();
                 dropped.column.appendCqlTo(builder);
 
-                if (!hasSingleColumnPrimaryKey || iter.hasNext())
+                if (!hasSingleColumnPrimaryKey || iterDropped.hasNext())
                     builder.append(',');
 
                 builder.newLine();
@@ -1297,7 +1297,7 @@ public class TableMetadata implements SchemaElement
         }
         else
         {
-            params.appendCqlTo(builder);
+            params.appendCqlTo(builder, isView());
         }
         builder.append(";");
     }
